@@ -97,6 +97,11 @@ public class Change extends ChangeCore {
         if ((null != capabilitiesToAdd && !capabilitiesToAdd.isEmpty()) || (null != capabilitiesToRemove && !capabilitiesToRemove.isEmpty()) || (null != capabilitiesToDrop && !capabilitiesToDrop.isEmpty())) {
             oxusr.changeCapabilities(ctx, usr, capabilitiesToAdd, capabilitiesToRemove, capabilitiesToDrop, auth);
         }
+
+        String personal = parseAndSetPersonal(parser);
+        if (null != personal) {
+            oxusr.changeMailAddressPersonal(ctx, usr, "NULL".equals(personal) ? null : personal, auth);
+        }
     }
 
     @Override
@@ -105,5 +110,6 @@ public class Change extends ChangeCore {
         setCapsToAdd(parser);
         setCapsToRemove(parser);
         setCapsToDrop(parser);
+        setPersonal(parser);
     }
 }
