@@ -73,6 +73,7 @@ import com.openexchange.admin.rmi.dataobjects.Credentials;
 import com.openexchange.admin.rmi.dataobjects.Database;
 import com.openexchange.admin.rmi.dataobjects.Filestore;
 import com.openexchange.admin.rmi.dataobjects.MaintenanceReason;
+import com.openexchange.admin.rmi.dataobjects.SchemaSelectStrategy;
 import com.openexchange.admin.rmi.dataobjects.User;
 import com.openexchange.admin.rmi.dataobjects.UserModuleAccess;
 import com.openexchange.admin.rmi.exceptions.ContextExistsException;
@@ -95,6 +96,7 @@ import com.openexchange.admin.storage.interfaces.OXUserStorageInterface;
 import com.openexchange.admin.storage.interfaces.OXUtilStorageInterface;
 import com.openexchange.admin.storage.sqlStorage.OXAdminPoolDBPoolExtension;
 import com.openexchange.admin.taskmanagement.TaskManager;
+import com.openexchange.admin.tools.AdminCache;
 import com.openexchange.admin.tools.DatabaseDataMover;
 import com.openexchange.admin.tools.FilestoreDataMover;
 import com.openexchange.caching.Cache;
@@ -231,7 +233,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
             throw new NoSuchContextException(e);
         }
 
-        LOGGER.debug(ctx+" - "+(null == capsToAdd ? "" : capsToAdd.toString())+" | "+(null == capsToRemove ? "" : capsToRemove.toString()));
+        LOGGER.debug(ctx + " - " + (null == capsToAdd ? "" : capsToAdd.toString()) + " | " + (null == capsToRemove ? "" : capsToRemove.toString()));
 
         try {
             if (!tool.existsContext(ctx)) {
@@ -265,7 +267,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public void change(final Context ctx, final Credentials credentials) throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         try {
             doNullCheck(ctx);
         } catch (final InvalidDataException e1) {
@@ -284,8 +286,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
         }
         LOGGER.debug(ctx.toString());
 
-        Context backup_ctx = null; // used for invalidating old login mappings in the cache
-
+        Set<String> loginMappings = null; // used for invalidating old login mappings in the cache
         try {
             if (!tool.existsContext(ctx)) {
                 throw new NoSuchContextException();
@@ -296,10 +297,10 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
             }
 
             // check if he wants to change the filestore id, if yes, make sure filestore with this id exists in the system
-            if(ctx.getFilestoreId()!=null) {
-                if(!tool.existsStore(ctx.getFilestoreId().intValue())){
-                    final InvalidDataException inde = new InvalidDataException("No such filestore with id "+ctx.getFilestoreId());
-                    LOGGER.error("",inde);
+            if (ctx.getFilestoreId() != null) {
+                if (!tool.existsStore(ctx.getFilestoreId().intValue())) {
+                    final InvalidDataException inde = new InvalidDataException("No such filestore with id " + ctx.getFilestoreId());
+                    LOGGER.error("", inde);
                     throw inde;
                 }
             }
@@ -315,7 +316,13 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
             }
 
             final OXContextStorageInterface oxcox = OXContextStorageInterface.getInstance();
-            backup_ctx = oxcox.getData(ctx);
+
+            // Check if login-mappings are supposed to be changed
+            if (null != ctx.getLoginMappings()) {
+                // Load old ones for invalidation purpose
+                loginMappings = oxcox.getLoginMappings(ctx);
+            }
+
             oxcox.change(ctx);
         } catch (final StorageException e) {
             LOGGER.error("", e);
@@ -331,27 +338,35 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
         try {
             final ContextStorage cs = ContextStorage.getInstance();
             cs.invalidateContext(ctx.getId().intValue());
-            final Set<String> loginMappings = backup_ctx.getLoginMappings();
             if (loginMappings != null && !loginMappings.isEmpty()) {
-                for (final String loginMapping : loginMappings) {
+                for (String loginMapping : loginMappings) {
                     cs.invalidateLoginInfo(loginMapping);
                 }
             }
         } catch (final OXException e) {
-            LOGGER.error("Error invalidating cached infos of context {} in context storage", ctx.getId(),e);
+            LOGGER.error("Error invalidating cached infos of context {} in context storage", ctx.getId(), e);
         }
     }
 
     @Override
     public Context create(final Context ctx, final User admin_user, final Credentials auth) throws RemoteException, StorageException, InvalidCredentialsException, InvalidDataException, ContextExistsException {
-        return createcommon(ctx, admin_user, null, null, auth);
+        return createcommon(ctx, admin_user, null, null, auth, getDefaultSchemaSelectStrategy());
     }
 
     @Override
-    public Context create(final Context ctx, final User admin_user,final String access_combination_name, final Credentials credentials)
-        throws RemoteException, StorageException,InvalidCredentialsException, InvalidDataException,    ContextExistsException {
+    public Context create(final Context ctx, final User admin_user, final Credentials auth, SchemaSelectStrategy schemaSelectStrategy) throws RemoteException, StorageException, InvalidCredentialsException, InvalidDataException, ContextExistsException {
+        return createcommon(ctx, admin_user, null, null, auth, schemaSelectStrategy);
+    }
 
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+    @Override
+    public Context create(final Context ctx, final User admin_user, final String access_combination_name, final Credentials credentials) throws RemoteException, StorageException, InvalidCredentialsException, InvalidDataException, ContextExistsException {
+        return create(ctx, admin_user, access_combination_name, credentials, getDefaultSchemaSelectStrategy());
+    }
+
+    @Override
+    public Context create(final Context ctx, final User admin_user, final String access_combination_name, final Credentials credentials, SchemaSelectStrategy schemaSelectStrategy) throws RemoteException, StorageException, InvalidCredentialsException, InvalidDataException, ContextExistsException {
+
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         // Resolve access rights by name
         try {
             doNullCheck(admin_user, access_combination_name);
@@ -366,20 +381,24 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
         LOGGER.debug("{} - {} - {} - {}", ctx, admin_user, access_combination_name, auth);
 
         UserModuleAccess access = ClientAdminThread.cache.getNamedAccessCombination(access_combination_name.trim(), true);
-        if(access==null){
+        if (access == null) {
             // no such access combination name defined in configuration
             // throw error!
-            throw new InvalidDataException("No such access combination name \""+access_combination_name.trim()+"\"");
+            throw new InvalidDataException("No such access combination name \"" + access_combination_name.trim() + "\"");
         }
         access = access.clone();
 
-        return createcommon(ctx, admin_user, null, access, auth);
+        return createcommon(ctx, admin_user, null, access, auth, schemaSelectStrategy);
     }
 
     @Override
-    public Context create(final Context ctx, final User admin_user,final UserModuleAccess access, final Credentials credentials)
-        throws RemoteException,StorageException, InvalidCredentialsException,InvalidDataException, ContextExistsException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+    public Context create(final Context ctx, final User admin_user, final UserModuleAccess access, final Credentials credentials) throws RemoteException, StorageException, InvalidCredentialsException, InvalidDataException, ContextExistsException {
+        return create(ctx, admin_user, access, credentials, getDefaultSchemaSelectStrategy());
+    }
+
+    @Override
+    public Context create(final Context ctx, final User admin_user, final UserModuleAccess access, final Credentials credentials, SchemaSelectStrategy schemaSelectStrategy) throws RemoteException, StorageException, InvalidCredentialsException, InvalidDataException, ContextExistsException {
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
 
         try {
             doNullCheck(admin_user, access);
@@ -388,13 +407,13 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
             throw e3;
         }
 
-        return createcommon(ctx, admin_user, null, access, auth);
+        return createcommon(ctx, admin_user, null, access, auth, schemaSelectStrategy);
 
     }
 
     @Override
     public void delete(final Context ctx, final Credentials credentials) throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, DatabaseUpdateException, InvalidDataException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         try {
             doNullCheck(ctx);
         } catch (final InvalidDataException e) {
@@ -463,7 +482,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
     }
 
     private void disable(final Context ctx, final MaintenanceReason reason, final Credentials credentials) throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException, NoSuchReasonException, OXContextException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         try {
             doNullCheck(ctx, reason);
             doNullCheck(reason.getId());
@@ -484,9 +503,11 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
             if (!tool.existsContext(ctx)) {
                 throw new NoSuchContextException();
             }
-            /*if (!tool.existsReason(reason_id)) {
-                throw new NoSuchReasonException();
-            }*/
+            /*
+             * if (!tool.existsReason(reason_id)) {
+             * throw new NoSuchReasonException();
+             * }
+             */
 
             // Trigger plugin extensions
             {
@@ -514,15 +535,17 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
                 ContextStorage.getInstance().invalidateContext(ctx.getId().intValue());
                 LOGGER.info("Context {} successfully invalidated", ctx.getId());
             } catch (final OXException e) {
-                LOGGER.error("Error invalidating context {} in ox context storage", ctx.getId(),e);
+                LOGGER.error("Error invalidating context {} in ox context storage", ctx.getId(), e);
             }
 
         } catch (final NoSuchContextException e) {
             LOGGER.error("", e);
             throw e;
-        /*} catch (final NoSuchReasonException e) {
-            log.error("", e);
-            throw e;*/
+            /*
+             * } catch (final NoSuchReasonException e) {
+             * log.error("", e);
+             * throw e;
+             */
         } catch (final StorageException e) {
             LOGGER.error("", e);
             throw e;
@@ -539,8 +562,8 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
     }
 
     private void disableAll(final MaintenanceReason reason, final Credentials credentials) throws RemoteException, StorageException, InvalidCredentialsException, InvalidDataException, NoSuchReasonException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
-        try{
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
+        try {
             doNullCheck(reason);
             doNullCheck(reason.getId());
         } catch (final InvalidDataException e1) {
@@ -552,11 +575,11 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
         final int reason_id = reason.getId();
         LOGGER.debug("{}", reason_id);
         try {
-//            if (!tool.existsReason(reason_id)) {
-//                throw new NoSuchReasonException();
-//            }
+            //            if (!tool.existsReason(reason_id)) {
+            //                throw new NoSuchReasonException();
+            //            }
             final OXContextStorageInterface oxcox = OXContextStorageInterface.getInstance();
-            if( ClientAdminThreadExtended.cache.isMasterAdmin(auth) ) {
+            if (ClientAdminThreadExtended.cache.isMasterAdmin(auth)) {
                 oxcox.disableAll(reason);
             } else {
                 // Trigger plugin extensions
@@ -577,14 +600,15 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
         } catch (final StorageException e) {
             LOGGER.error("", e);
             throw e;
-//        } catch (final NoSuchReasonException e) {
-//            log.error("", e);
-//            throw e;
+            //        } catch (final NoSuchReasonException e) {
+            //            log.error("", e);
+            //            throw e;
         }
 
         // Clear context cache
         // CACHE
-        final CacheService cacheService = AdminServiceRegistry.getInstance().getService(CacheService.class);;
+        final CacheService cacheService = AdminServiceRegistry.getInstance().getService(CacheService.class);
+        ;
         if (null != cacheService) {
             try {
                 final Cache cache = cacheService.getCache("Context");
@@ -598,7 +622,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public void enable(final Context ctx, final Credentials credentials) throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         try {
             doNullCheck(ctx);
         } catch (final InvalidDataException e1) {
@@ -648,19 +672,19 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
         try {
             ContextStorage.getInstance().invalidateContext(ctx.getId().intValue());
         } catch (final OXException e) {
-            LOGGER.error("Error invalidating context {} in ox context storage", ctx.getId(),e);
+            LOGGER.error("Error invalidating context {} in ox context storage", ctx.getId(), e);
         }
     }
 
     @Override
     public void enableAll(final Credentials credentials) throws RemoteException, StorageException, InvalidCredentialsException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
 
         new BasicAuthenticator(context).doAuthentication(auth);
 
         try {
             final OXContextStorageInterface oxcox = OXContextStorageInterface.getInstance();
-            if( ClientAdminThreadExtended.cache.isMasterAdmin(auth) ) {
+            if (ClientAdminThreadExtended.cache.isMasterAdmin(auth)) {
                 oxcox.enableAll();
             } else {
                 // Trigger plugin extensions
@@ -685,7 +709,8 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
         // Clear context cache
         // CACHE
-        final CacheService cacheService = AdminServiceRegistry.getInstance().getService(CacheService.class);;
+        final CacheService cacheService = AdminServiceRegistry.getInstance().getService(CacheService.class);
+        ;
         if (null != cacheService) {
             try {
                 final Cache cache = cacheService.getCache("Context");
@@ -699,10 +724,10 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public Context[] getData(final Context[] ctxs, final Credentials credentials) throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         try {
             try {
-                doNullCheck((Object[])ctxs);
+                doNullCheck((Object[]) ctxs);
             } catch (final InvalidDataException e1) {
                 LOGGER.error("One of the given arguments for getData is null", e1);
                 throw e1;
@@ -763,12 +788,12 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public Context getData(final Context ctx, final Credentials auth) throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException {
-        return getData(new Context[]{ctx}, auth)[0];
+        return getData(new Context[] { ctx }, auth)[0];
     }
 
     @Override
     public Context[] list(final String search_pattern, final Credentials credentials) throws RemoteException, StorageException, InvalidCredentialsException, InvalidDataException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         try {
             doNullCheck(search_pattern);
         } catch (final InvalidDataException e) {
@@ -785,8 +810,8 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
             Filter<Context, Context> loader = null;
             Filter<Integer, Integer> filter = null;
-            final ArrayList<Filter<Context, Context>> loaderFilter = new ArrayList<Filter<Context,Context>>();
-            final ArrayList<Filter<Integer, Integer>> contextFilter = new ArrayList<Filter<Integer,Integer>>();
+            final ArrayList<Filter<Context, Context>> loaderFilter = new ArrayList<Filter<Context, Context>>();
+            final ArrayList<Filter<Integer, Integer>> contextFilter = new ArrayList<Filter<Integer, Integer>>();
 
             final PluginInterfaces pluginInterfaces = PluginInterfaces.getInstance();
             if (null != pluginInterfaces) {
@@ -803,7 +828,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
                             loaderFilter.add(loader);
                         }
                     } catch (final PluginException e) {
-                        LOGGER.error("Error while calling method list of plugin {}", bundlename,e);
+                        LOGGER.error("Error while calling method list of plugin {}", bundlename, e);
                         throw StorageException.wrapForRMI(e);
                     }
                 }
@@ -827,7 +852,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public Context[] listByDatabase(final Database db, final Credentials credentials) throws RemoteException, StorageException, InvalidCredentialsException, InvalidDataException, NoSuchDatabaseException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         try {
             doNullCheck(db);
         } catch (final InvalidDataException e) {
@@ -844,7 +869,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
         }
         LOGGER.debug(db.toString());
         try {
-            if( !tool.existsDatabase(db.getId()) ) {
+            if (!tool.existsDatabase(db.getId())) {
                 throw new NoSuchDatabaseException();
             }
             final OXContextStorageInterface oxcox = OXContextStorageInterface.getInstance();
@@ -866,7 +891,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public Context[] listByFilestore(final Filestore filestore, final Credentials credentials) throws RemoteException, StorageException, InvalidCredentialsException, InvalidDataException, NoSuchFilestoreException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         try {
             doNullCheck(filestore);
             doNullCheck(filestore.getId());
@@ -879,7 +904,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
         LOGGER.debug(filestore.toString());
         try {
-            if( !tool.existsStore(filestore.getId()) ) {
+            if (!tool.existsStore(filestore.getId())) {
                 throw new NoSuchFilestoreException();
             }
             final OXContextStorageInterface oxcox = OXContextStorageInterface.getInstance();
@@ -908,9 +933,9 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
     }
 
     private int moveContextDatabase(final Context ctx, final Database db, final MaintenanceReason reason, final Credentials credentials) throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException, DatabaseUpdateException, OXContextException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
-        try{
-            doNullCheck(ctx,db,reason);
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
+        try {
+            doNullCheck(ctx, db, reason);
             doNullCheck(reason.getId());
         } catch (final InvalidDataException e1) {
             LOGGER.error("Invalid data sent by client!", e1);
@@ -931,16 +956,18 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
             throw new StorageException(e);
         }
         final int reason_id = reason.getId();
-            LOGGER.debug("{} - {} - {}", ctx, db, reason_id);
+        LOGGER.debug("{} - {} - {}", ctx, db, reason_id);
         try {
-            /*if (!tool.existsReason(reason_id)) {
-                // FIXME: Util in context???
-                throw new OXContextException(OXUtilException.NO_SUCH_REASON);
-            }*/
+            /*
+             * if (!tool.existsReason(reason_id)) {
+             * // FIXME: Util in context???
+             * throw new OXContextException(OXUtilException.NO_SUCH_REASON);
+             * }
+             */
             if (!tool.existsContext(ctx)) {
                 throw new NoSuchContextException();
             }
-            if( tool.checkAndUpdateSchemaIfRequired(ctx) ) {
+            if (tool.checkAndUpdateSchemaIfRequired(ctx)) {
                 throw new DatabaseUpdateException("Database is locked or is now beeing updated, please try again later");
             }
             if (!tool.isContextEnabled(ctx)) {
@@ -982,7 +1009,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
     }
 
     private int moveContextFilestore(final Context ctx, final Filestore dst_filestore, final MaintenanceReason reason, final Credentials credentials) throws InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException, NoSuchFilestoreException, OXContextException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         try {
             doNullCheck(ctx, dst_filestore, reason);
             doNullCheck(dst_filestore.getId(), reason.getId());
@@ -1080,39 +1107,38 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
     }
 
     @Override
-    protected Context createmaincall(final Context ctx, final User admin_user, final Database db, final UserModuleAccess access, final Credentials auth) throws StorageException, InvalidDataException {
+    protected Context createmaincall(final Context ctx, final User admin_user, final Database db, final UserModuleAccess access, final Credentials auth, SchemaSelectStrategy schemaSelectStrategy) throws StorageException, InvalidDataException, ContextExistsException {
         validateloginmapping(ctx);
-        final OXContextStorageInterface oxcox = OXContextStorageInterface.getInstance();
+        OXContextStorageInterface oxcox = OXContextStorageInterface.getInstance();
+
+        String DEFAULT_ACCESS_COMBINATION_NAME = ClientAdminThreadExtended.cache.getProperties().getProp("NEW_CONTEXT_DEFAULT_ACCESS_COMBINATION_NAME", "NOT_DEFINED");
 
-        final String DEFAULT_ACCESS_COMBINATION_NAME = ClientAdminThreadExtended.cache.getProperties().getProp("NEW_CONTEXT_DEFAULT_ACCESS_COMBINATION_NAME", "NOT_DEFINED");
         // If not defined or access combination name does NOT exist, use hardcoded fallback!
-        UserModuleAccess createaccess = null;
-        if( access == null ) {
-            if(DEFAULT_ACCESS_COMBINATION_NAME.equals("NOT_DEFINED") || ClientAdminThread.cache.getNamedAccessCombination(DEFAULT_ACCESS_COMBINATION_NAME, true) == null){
-                createaccess = ClientAdminThread.cache.getDefaultUserModuleAccess();
-            }else{
-                createaccess = ClientAdminThread.cache.getNamedAccessCombination(DEFAULT_ACCESS_COMBINATION_NAME, true);
+        UserModuleAccess createaccess;
+        if (access == null) {
+            if (DEFAULT_ACCESS_COMBINATION_NAME.equals("NOT_DEFINED") || ClientAdminThread.cache.getNamedAccessCombination(DEFAULT_ACCESS_COMBINATION_NAME, true) == null) {
+                createaccess = AdminCache.getDefaultUserModuleAccess().clone();
+            } else {
+                createaccess = ClientAdminThread.cache.getNamedAccessCombination(DEFAULT_ACCESS_COMBINATION_NAME, true).clone();
             }
         } else {
-            createaccess = access;
+            createaccess = access.clone();
         }
-        createaccess = createaccess.clone();
 
-        Context ret = ctx;
-        ret = oxcox.create(ret, admin_user, createaccess);
-        if( isAnyPluginLoaded() ) {
-            final PluginInterfaces pluginInterfaces = PluginInterfaces.getInstance();
+        Context ret = oxcox.create(ctx, admin_user, createaccess, schemaSelectStrategy == null ? getDefaultSchemaSelectStrategy() : schemaSelectStrategy);
+        if (isAnyPluginLoaded()) {
+            PluginInterfaces pluginInterfaces = PluginInterfaces.getInstance();
             if (null != pluginInterfaces) {
-                for (final OXContextPluginInterface contextInterface : pluginInterfaces.getContextPlugins().getServiceList()) {
+                for (OXContextPluginInterface contextInterface : pluginInterfaces.getContextPlugins().getServiceList()) {
                     try {
                         ret = contextInterface.postCreate(ret, admin_user, createaccess, auth);
-                    } catch (final PluginException e) {
-                        LOGGER.error("",e);
+                    } catch (PluginException e) {
+                        LOGGER.error("", e);
                         // callPluginMethod delete may fail here for what ever reason.
                         // this must not prevent us from cleaning up the rest
                         try {
                             contextInterface.delete(ctx, auth);
-                        } catch (final Exception e1) {
+                        } catch (Exception e1) {
                             LOGGER.error("", e);
                         }
                         oxcox.delete(ret);
@@ -1121,6 +1147,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
                 }
             }
         }
+
         return ret;
     }
 
@@ -1129,8 +1156,8 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
         final String login_regexp = ClientAdminThreadExtended.cache.getProperties().getProp("CHECK_CONTEXT_LOGIN_MAPPING_REGEXP", "[$%\\.+a-zA-Z0-9_-]");
         if (null != loginMappings) {
             for (final String mapping : loginMappings) {
-                final String illegal = mapping.replaceAll(login_regexp,"");
-                if( illegal.length() > 0 ) {
+                final String illegal = mapping.replaceAll(login_regexp, "");
+                if (illegal.length() > 0) {
                     throw new InvalidDataException("Illegal chars: \"" + illegal + "\"" + " in login mapping");
                 }
             }
@@ -1139,11 +1166,11 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     private StringBuilder builduppath(final String ctxdir, final URI uri) {
         final StringBuilder src = new StringBuilder(uri.getPath());
-        if (src.length() == 0 || src.charAt(src.length()-1) != '/') {
+        if (src.length() == 0 || src.charAt(src.length() - 1) != '/') {
             src.append('/');
         }
         src.append(ctxdir);
-        if (src.charAt(src.length()-1) == '/') {
+        if (src.charAt(src.length() - 1) == '/') {
             src.deleteCharAt(src.length() - 1);
         }
         return src;
@@ -1151,8 +1178,8 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public void changeModuleAccess(final Context ctx, final UserModuleAccess access, final Credentials credentials)
-        throws RemoteException,InvalidCredentialsException, NoSuchContextException,    StorageException, InvalidDataException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException {
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
 
         try {
             doNullCheck(access);
@@ -1211,9 +1238,9 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public void changeModuleAccess(final Context ctx, final String access_combination_name, final Credentials credentials)
-        throws RemoteException,InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException {
+        throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException {
 
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
 
         try {
             doNullCheck(access_combination_name);
@@ -1246,7 +1273,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
             if (null == accessAdmin || null == accessUser) {
                 // no such access combination name defined in configuration
                 // throw error!
-                throw new InvalidDataException("No such access combination name \""+access_combination_name.trim()+"\"");
+                throw new InvalidDataException("No such access combination name \"" + access_combination_name.trim() + "\"");
             }
             accessAdmin = accessAdmin.clone();
             accessUser = accessUser.clone();
@@ -1283,14 +1310,13 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
      */
     @Override
     public void downgrade(final Context ctx, final Credentials credentials) throws
-        RemoteException, InvalidCredentialsException,NoSuchContextException,
-        StorageException, DatabaseUpdateException,InvalidDataException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        RemoteException, InvalidCredentialsException, NoSuchContextException,
+        StorageException, DatabaseUpdateException, InvalidDataException {
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         try {
             doNullCheck(ctx);
         } catch (final InvalidDataException e1) {
-            final InvalidDataException invalidDataException
-                = new InvalidDataException("Context is invalid");
+            final InvalidDataException invalidDataException = new InvalidDataException("Context is invalid");
             LOGGER.error("", invalidDataException);
             throw invalidDataException;
         }
@@ -1333,14 +1359,14 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
         try {
             ContextStorage.getInstance().invalidateContext(ctx.getId().intValue());
         } catch (final OXException e) {
-            LOGGER.error("Error invalidating context {} in ox context storage", ctx.getId(),e);
+            LOGGER.error("Error invalidating context {} in ox context storage", ctx.getId(), e);
         }
     }
 
     @Override
     public String getAccessCombinationName(final Context ctx, final Credentials credentials)
-        throws RemoteException, InvalidCredentialsException,NoSuchContextException, StorageException, InvalidDataException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException {
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
 
         // Resolve admin user and get the module access from db and query cache for access combination name
         try {
@@ -1393,8 +1419,8 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public UserModuleAccess getModuleAccess(final Context ctx, final Credentials credentials)
-        throws RemoteException, InvalidCredentialsException,NoSuchContextException, StorageException, InvalidDataException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        throws RemoteException, InvalidCredentialsException, NoSuchContextException, StorageException, InvalidDataException {
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
 
         try {
             doNullCheck(ctx);
@@ -1463,7 +1489,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
                     retval = oxctx.getData(ctxs, auth);
                     addExtensionToContext(ctxs, retval, bundlename);
                 } catch (final PluginException e) {
-                    LOGGER.error("Error while calling method list of plugin {}", bundlename,e);
+                    LOGGER.error("Error while calling method list of plugin {}", bundlename, e);
                     throw StorageException.wrapForRMI(e);
                 }
             }
@@ -1489,7 +1515,7 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public int getAdminId(final Context ctx, final Credentials credentials) throws RemoteException, InvalidCredentialsException, StorageException, NoSuchContextException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
 
         new BasicAuthenticator(context).doAuthentication(auth);
 
@@ -1517,10 +1543,10 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
 
     @Override
     public boolean exists(final Context ctx, final Credentials credentials) throws RemoteException, InvalidDataException, StorageException, InvalidCredentialsException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        final Credentials auth = credentials == null ? new Credentials("", "") : credentials;
         new BasicAuthenticator(context).doAuthentication(auth);
 
-        if(ctx == null) {
+        if (ctx == null) {
             throw new InvalidDataException("Given context is invalid");
         }
 
@@ -1539,9 +1565,9 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
             }
         }
 
-        if( null != ctx.getId() ) {
-        return tool.existsContext(ctx);
-        } else if( null != ctx.getName() ) {
+        if (null != ctx.getId()) {
+            return tool.existsContext(ctx);
+        } else if (null != ctx.getName()) {
             return tool.existsContextName(ctx.getName());
         } else {
             throw new InvalidDataException("neither id or name is set in supplied context object");
@@ -1553,6 +1579,16 @@ public class OXContext extends OXContextCommonImpl implements OXContextInterface
         return exists(ctx, credentials);
     }
 
+    @Override
+    public void checkCountsConsistency(boolean checkDatabaseCounts, boolean checkFilestoreCounts, Credentials credentials) throws RemoteException, StorageException, InvalidCredentialsException {
+        Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        new BasicAuthenticator(context).doAuthentication(auth);
+
+        LOGGER.debug("Checking consistency for counters");
+
+        OXContextStorageInterface.getInstance().checkCountsConsistency(checkDatabaseCounts, checkFilestoreCounts);
+    }
+
     /** Check for an empty string */
     private static boolean isEmpty(final String string) {
         if (null == string) {
