@@ -51,6 +51,7 @@ package com.openexchange.admin.storage.interfaces;
 
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
+import java.sql.Connection;
 import java.util.List;
 import java.util.Set;
 import com.openexchange.admin.daemons.ClientAdminThreadExtended;
@@ -59,8 +60,10 @@ import com.openexchange.admin.rmi.dataobjects.Credentials;
 import com.openexchange.admin.rmi.dataobjects.Database;
 import com.openexchange.admin.rmi.dataobjects.Filestore;
 import com.openexchange.admin.rmi.dataobjects.MaintenanceReason;
+import com.openexchange.admin.rmi.dataobjects.SchemaSelectStrategy;
 import com.openexchange.admin.rmi.dataobjects.User;
 import com.openexchange.admin.rmi.dataobjects.UserModuleAccess;
+import com.openexchange.admin.rmi.exceptions.ContextExistsException;
 import com.openexchange.admin.rmi.exceptions.InvalidDataException;
 import com.openexchange.admin.rmi.exceptions.StorageException;
 import com.openexchange.admin.tools.AdminCacheExtended;
@@ -87,7 +90,7 @@ public abstract class OXContextStorageInterface {
 
     /**
      * Creates a new instance implementing the group storage interface.
-     * 
+     *
      * @return an instance implementing the group storage interface.
      * @throws com.openexchange.admin.rmi.exceptions.StorageException Storage exception
      */
@@ -159,6 +162,15 @@ public abstract class OXContextStorageInterface {
      */
     public abstract void changeStorageData(final Context ctx) throws StorageException;
 
+    /**
+     * Gets the login mappings for specified context.
+     *
+     * @param ctx The context for which to load the mappings
+     * @return The available login mappings
+     * @throws StorageException If login mappings cannot be returned
+     */
+    public abstract Set<String> getLoginMappings(Context ctx) throws StorageException;
+
     /**
      * @param ctx
      * @return a context object
@@ -215,9 +227,12 @@ public abstract class OXContextStorageInterface {
      * @param ctx
      * @param admin_user
      * @param access
-     * @throws StorageException
+     * @param schemaSelectStrategy
+     * @throws StorageException If a general storage error occurs
+     * @throws InvalidDataException If there is already a context with the same name
+     * @throws ContextExistsException If there is already a context with the same context identifier
      */
-    public abstract Context create(final Context ctx, final User admin_user, final UserModuleAccess access) throws StorageException, InvalidDataException;
+    public abstract Context create(final Context ctx, final User admin_user, final UserModuleAccess access, SchemaSelectStrategy schemaSelectStrategy) throws StorageException, InvalidDataException, ContextExistsException;
 
     /**
      * @param ctx
@@ -304,7 +319,7 @@ public abstract class OXContextStorageInterface {
 
     /**
      * This method deletes all inaccessible data in a context.
-     * 
+     *
      * @param ctx Context.
      * @throws StorageException if some problem occurs.
      */
@@ -327,7 +342,7 @@ public abstract class OXContextStorageInterface {
 
     /**
      * Updates the context references after a replay
-     * 
+     *
      * @param sourceSchema The source schema
      * @param targetSchema The target schema
      * @param targetClusterId The target cluster identifier
@@ -337,11 +352,30 @@ public abstract class OXContextStorageInterface {
 
     /**
      * Create a new database schema
-     * 
+     *
      * @param targetClusterId The identifier of the target cluster
      * @return The name of the new database schema
      * @throws StorageException
      */
     public abstract String createSchema(int targetClusterId) throws StorageException;
 
+    /**
+     * Checks the consistencies for the count tables
+     *
+     * @param configCon The connection to the ConfigDb
+     * @param checkDatabaseCounts Whether to check the counts related to context to database/schema associations
+     * @param checkFilestoreCounts Whether to check the counts related to context to filestore associations
+     * @throws StorageException If check fails
+     */
+    public abstract void checkCountsConsistency(Connection configCon, boolean checkDatabaseCounts, boolean checkFilestoreCounts) throws StorageException;
+
+    /**
+     * Checks the consistencies for the count tables
+     *
+     * @param checkDatabaseCounts Whether to check the counts related to context to database/schema associations
+     * @param checkFilestoreCounts Whether to check the counts related to context to filestore associations
+     * @throws StorageException If check fails
+     */
+    public abstract void checkCountsConsistency(boolean checkDatabaseCounts, boolean checkFilestoreCounts) throws StorageException;
+
 }
