@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -71,14 +71,8 @@ public class DBWeightComparator implements Comparator<DatabaseHandle> {
 
     @Override
     public int compare(DatabaseHandle db1, DatabaseHandle db2) {
-        int missingUnits1 = getMissingUnits(db1);
-        if (isFull(db1)) {
-            missingUnits1 = Integer.MIN_VALUE;
-        }
-        int missingUnits2 = getMissingUnits(db2);
-        if (isFull(db2)) {
-            missingUnits2 = Integer.MIN_VALUE;
-        }
+        int missingUnits1 = isFull(db1) ? Integer.MIN_VALUE : getMissingUnits(db1);
+        int missingUnits2 = isFull(db2) ? Integer.MIN_VALUE : getMissingUnits(db2);
         return I(missingUnits1).compareTo(I(missingUnits2));
     }
 
