@@ -56,6 +56,7 @@ import com.openexchange.admin.rmi.dataobjects.Context;
 import com.openexchange.admin.rmi.dataobjects.Credentials;
 import com.openexchange.admin.rmi.dataobjects.Database;
 import com.openexchange.admin.rmi.dataobjects.Filestore;
+import com.openexchange.admin.rmi.dataobjects.SchemaSelectStrategy;
 import com.openexchange.admin.rmi.dataobjects.User;
 import com.openexchange.admin.rmi.dataobjects.UserModuleAccess;
 import com.openexchange.admin.rmi.exceptions.ContextExistsException;
@@ -165,6 +166,61 @@ public interface OXContextInterface extends Remote {
      */
     public Context create(final Context ctx, final User admin_user, UserModuleAccess access,final Credentials auth) throws RemoteException, StorageException, InvalidCredentialsException,InvalidDataException, ContextExistsException;
 
+    /**
+     * Create a new context.
+     *
+     * If setFilestoreId() or setWriteDatabase() has been used in the given context object, the context will be created
+     * in the corresponding database or filestore.
+     * The assigned limits to the database/filestore are ignored, though.
+     *
+     * @param ctx A new Context object, this should not have been used before or a one returned from a previous call to this API.
+     * @param admin_user User data of administrative user account for this context
+     * @param auth Credentials for authenticating against server.
+     * @param schemaSelectStrategy SchemaSelectStrategy to define how to select the schema where the context will be created.
+     *
+     * @return Context object.
+     * @throws com.openexchange.admin.rmi.exceptions.InvalidCredentialsException When the supplied credentials were not correct or invalid.
+     * @throws com.openexchange.admin.rmi.exceptions.InvalidDataException If the data sent within the method contained invalid data.
+     * @throws RemoteException General RMI Exception
+     * @throws StorageException When an error in the subsystems occurred.
+     * @throws ContextExistsException
+     */
+    public Context create(final Context ctx, final User admin_user, final Credentials auth, SchemaSelectStrategy schemaSelectStrategy) throws RemoteException, StorageException, InvalidCredentialsException,InvalidDataException, ContextExistsException;
+
+    /**
+     * Create a new context! Given access combination name will be used for admin module access rights!
+     * @param ctx Context object
+     * @param admin_user User data of administrative user account for this context
+     * @param access_combination_name String Access combination name!
+     * @param auth Credentials for authenticating against server.
+     * @param schemaSelectStrategy SchemaSelectStrategy to define how to select the schema where the context will be created.
+     *
+     * @return A new Context object, this should not have been used before or a one returned from a previous call to this API.
+     * @throws com.openexchange.admin.rmi.exceptions.InvalidCredentialsException When the supplied credentials were not correct or invalid.
+     * @throws com.openexchange.admin.rmi.exceptions.InvalidDataException If the data sent within the method contained invalid data.
+     * @throws RemoteException General RMI Exception
+     * @throws StorageException When an error in the subsystems occurred.
+     * @throws ContextExistsException
+     */
+    public Context create(final Context ctx, final User admin_user, String access_combination_name,final Credentials auth, SchemaSelectStrategy schemaSelectStrategy) throws RemoteException, StorageException, InvalidCredentialsException,InvalidDataException, ContextExistsException;
+
+    /**
+     * Create a new context! Given access rights be used for admin!
+     * @param ctx Context object
+     * @param admin_user User data of administrative user account for this context
+     * @param access UserModuleAccess Access rights!
+     * @param auth Credentials for authenticating against server.
+     * @param schemaSelectStrategy SchemaSelectStrategy to define how to select the schema where the context will be created.
+     *
+     * @return A new Context object, this should not have been used before or a one returned from a previous call to this API.
+     * @throws com.openexchange.admin.rmi.exceptions.InvalidCredentialsException When the supplied credentials were not correct or invalid.
+     * @throws com.openexchange.admin.rmi.exceptions.InvalidDataException If the data sent within the method contained invalid data.
+     * @throws RemoteException General RMI Exception
+     * @throws StorageException When an error in the subsystems occurred.
+     * @throws ContextExistsException
+     */
+    public Context create(final Context ctx, final User admin_user, UserModuleAccess access,final Credentials auth, SchemaSelectStrategy schemaSelectStrategy) throws RemoteException, StorageException, InvalidCredentialsException,InvalidDataException, ContextExistsException;
+
 
     /**
      * Delete a context.<br>
@@ -374,7 +430,7 @@ public interface OXContextInterface extends Remote {
      *
      * Change storage data informations - Change filestore infos for context. Normally NO need to change!
      *
-     * @param ctx A new Context object, this should not have been used before or a one returned from a previous call to this API. Beside the context ID 
+     * @param ctx A new Context object, this should not have been used before or a one returned from a previous call to this API. Beside the context ID
      * or name for identifying the context itself the object should only contain those field which need to be changed.
      * @param auth
      * @throws RemoteException
@@ -530,7 +586,7 @@ public interface OXContextInterface extends Remote {
 
     /**
      * Determines the user ID of the admin user for a given context
-     * 
+     *
      * @param ctx A new Context object, this should not have been used before or a one returned from a previous call to this API. This context will be used for determining the userId of the admin.
      * @param auth Credentials for authenticating against the server.
      * @return The userId of the admin user
@@ -543,7 +599,7 @@ public interface OXContextInterface extends Remote {
 
     /**
      * Determines whether a context already exists.
-     * 
+     *
      * @param ctx A new Context object, this should not have been used before or a one returned from a previous call to this API.
      * @param auth Credentials for authenticating against the server.
      * @return Whether the given context exists or not
@@ -552,7 +608,7 @@ public interface OXContextInterface extends Remote {
 
     /**
      * Determines whether a context already exists.
-     * 
+     *
      * @param ctx A new Context object, this should not have been used before or a one returned from a previous call to this API.
      * @param auth Credentials for authenticating against the server.
      * @return Whether the given context exists or not
@@ -560,4 +616,17 @@ public interface OXContextInterface extends Remote {
      */
     @Deprecated
     public boolean checkExists(Context ctx, Credentials auth) throws RemoteException, InvalidDataException, StorageException, InvalidCredentialsException;
+
+    /**
+     * Checks the consistencies for the count tables
+     *
+     * @param checkDatabaseCounts Whether to check the counts related to context to database/schema associations
+     * @param checkFilestoreCounts Whether to check the counts related to context to filestore associations
+     * @param auth Credentials for authenticating against server
+     * @throws com.openexchange.admin.rmi.exceptions.StorageException When an error in the subsystems occurred
+     * @throws com.openexchange.admin.rmi.exceptions.InvalidCredentialsException When the supplied credentials were not correct or invalid
+     * @throws RemoteException General RMI Exception
+     */
+    public void checkCountsConsistency(boolean checkDatabaseCounts, boolean checkFilestoreCounts, Credentials auth) throws RemoteException, StorageException, InvalidCredentialsException;
+
 }
