@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,6 +52,7 @@ package com.openexchange.admin.rmi.impl;
 import org.osgi.framework.BundleContext;
 import com.openexchange.admin.daemons.ClientAdminThread;
 import com.openexchange.admin.plugins.BasicAuthenticatorPluginInterface;
+import com.openexchange.admin.properties.AdminProperties;
 import com.openexchange.admin.rmi.dataobjects.Context;
 import com.openexchange.admin.rmi.dataobjects.Credentials;
 import com.openexchange.admin.rmi.exceptions.InvalidCredentialsException;
@@ -105,7 +106,15 @@ public class BasicAuthenticator extends OXCommonImpl {
      * @throws InvalidCredentialsException
      */
     public void doAuthentication(final Credentials authdata) throws InvalidCredentialsException{
+        boolean autoLowerCase = cache.getProperties().getUserProp(AdminProperties.User.AUTO_LOWERCASE, false);
+        if (autoLowerCase && null != authdata.getLogin()) {
+            authdata.setLogin(authdata.getLogin().toLowerCase());
+        }
+
         final Credentials master = ClientAdminThread.cache.getMasterCredentials();
+        if (autoLowerCase && null != master.getLogin()) {
+            master.setLogin(master.getLogin().toLowerCase());
+        }
 
         boolean doPluginAuth = false;
         if (cache.masterAuthenticationDisabled() || (authdata != null && master != null && !master.getLogin().equals(authdata.getLogin()))) {
@@ -160,6 +169,11 @@ public class BasicAuthenticator extends OXCommonImpl {
     public void doAuthentication(final Credentials authdata,final Context ctx) throws InvalidCredentialsException, StorageException, InvalidDataException{
         contextcheck(ctx);
 
+        boolean autoLowerCase = cache.getProperties().getUserProp(AdminProperties.User.AUTO_LOWERCASE, false);
+        if (autoLowerCase) {
+            authdata.setLogin(authdata.getLogin().toLowerCase());
+        }
+
         // only do context check, if we have not already admin creds in our cache for given context
         // ATTENTION: It is correct that we don't throw a now such context exception here because we won't
         // give an opportunity to indirectly check for contexts here
@@ -195,6 +209,11 @@ public class BasicAuthenticator extends OXCommonImpl {
     public void doUserAuthentication(final Credentials authdata,final Context ctx) throws InvalidCredentialsException, StorageException, InvalidDataException{
         contextcheck(ctx);
 
+        boolean autoLowerCase = cache.getProperties().getUserProp(AdminProperties.User.AUTO_LOWERCASE, false);
+        if (autoLowerCase) {
+            authdata.setLogin(authdata.getLogin().toLowerCase());
+        }
+
         if (!OXToolStorageInterface.getInstance().existsContext(ctx)) {
             final InvalidCredentialsException invalidCredentialsException = new InvalidCredentialsException(
                     "Authentication failed for user " + authdata.getLogin());
