@@ -46,6 +46,7 @@
  *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  */
+
 package com.openexchange.admin.rmi.impl;
 
 import org.osgi.framework.BundleContext;
@@ -54,6 +55,7 @@ import com.openexchange.admin.plugins.PluginException;
 import com.openexchange.admin.rmi.dataobjects.Context;
 import com.openexchange.admin.rmi.dataobjects.Credentials;
 import com.openexchange.admin.rmi.dataobjects.Database;
+import com.openexchange.admin.rmi.dataobjects.SchemaSelectStrategy;
 import com.openexchange.admin.rmi.dataobjects.User;
 import com.openexchange.admin.rmi.dataobjects.UserModuleAccess;
 import com.openexchange.admin.rmi.exceptions.ContextExistsException;
@@ -65,7 +67,6 @@ import com.openexchange.admin.services.PluginInterfaces;
 import com.openexchange.admin.storage.interfaces.OXToolStorageInterface;
 import com.openexchange.admin.tools.GenericChecks;
 
-
 public abstract class OXContextCommonImpl extends OXCommonImpl {
 
     private final static org.slf4j.Logger LOGGER = org.slf4j.LoggerFactory.getLogger(OXContextCommonImpl.class);
@@ -93,7 +94,7 @@ public abstract class OXContextCommonImpl extends OXCommonImpl {
                 }
             }
 
-            if( ret == null || ( ret != null && ret.booleanValue())  ) {
+            if (ret == null || (ret != null && ret.booleanValue())) {
                 if (!ctx.mandatoryCreateMembersSet()) {
                     throw new InvalidDataException("Mandatory fields in context not set: " + ctx.getUnsetMembers());
                 }
@@ -104,14 +105,6 @@ public abstract class OXContextCommonImpl extends OXCommonImpl {
             throw StorageException.wrapForRMI(e);
         }
 
-        if (tool.existsContext(ctx)) {
-            throw new ContextExistsException("Context already exists!");
-        }
-
-        if(ctx.getName()!=null && tool.existsContextName(ctx.getName())){
-            throw new InvalidDataException("Context " + ctx.getName() + " already exists!");
-        }
-
         try {
             if (!admin_user.mandatoryCreateMembersSet()) {
                 throw new InvalidDataException("Mandatory fields in admin user not set: " + admin_user.getUnsetMembers());
@@ -123,11 +116,15 @@ public abstract class OXContextCommonImpl extends OXCommonImpl {
         GenericChecks.checkValidMailAddress(admin_user.getPrimaryEmail());
     }
 
-    protected abstract Context createmaincall(final Context ctx, final User admin_user, Database db, UserModuleAccess access, final Credentials auth) throws StorageException, InvalidDataException;
+    protected abstract Context createmaincall(final Context ctx, final User admin_user, Database db, UserModuleAccess access, final Credentials auth, SchemaSelectStrategy schemaSelectStrategy) throws StorageException, InvalidDataException, ContextExistsException;
 
-    protected Context createcommon(final Context ctx, final User admin_user, final Database db, final UserModuleAccess access, final Credentials auth) throws InvalidCredentialsException, ContextExistsException, InvalidDataException, StorageException {
-        try{
-            doNullCheck(ctx,admin_user);
+    protected SchemaSelectStrategy getDefaultSchemaSelectStrategy() {
+        return SchemaSelectStrategy.getDefault();
+    }
+
+    protected Context createcommon(final Context ctx, final User admin_user, final Database db, final UserModuleAccess access, final Credentials auth, SchemaSelectStrategy schemaSelectStrategy) throws InvalidCredentialsException, ContextExistsException, InvalidDataException, StorageException {
+        try {
+            doNullCheck(ctx, admin_user);
         } catch (final InvalidDataException e1) {
             final InvalidDataException invalidDataException = new InvalidDataException("Context or user not correct");
             LOGGER.error("", invalidDataException);
@@ -141,14 +138,14 @@ public abstract class OXContextCommonImpl extends OXCommonImpl {
         try {
             final OXToolStorageInterface tool = OXToolStorageInterface.getInstance();
             Context ret = ctx;
-            if( isAnyPluginLoaded() ) {
+            if (isAnyPluginLoaded()) {
                 final PluginInterfaces pluginInterfaces = PluginInterfaces.getInstance();
                 if (null != pluginInterfaces) {
                     for (final OXContextPluginInterface contextInterface : pluginInterfaces.getContextPlugins().getServiceList()) {
                         try {
                             ret = contextInterface.preCreate(ret, admin_user, auth);
                         } catch (PluginException e) {
-                            LOGGER.error("",e);
+                            LOGGER.error("", e);
                             throw StorageException.wrapForRMI(e);
                         }
                     }
@@ -160,7 +157,7 @@ public abstract class OXContextCommonImpl extends OXCommonImpl {
             // Ensure context identifier is contained in login mappings
             {
                 final String sContextId = ret.getIdAsString();
-                if (null !=sContextId) {
+                if (null != sContextId) {
                     ret.addLoginMapping(sContextId);
                 }
             }
@@ -174,19 +171,17 @@ public abstract class OXContextCommonImpl extends OXCommonImpl {
                 }
             }
 
-            final Context retval = createmaincall(ret, admin_user, db, access,auth);
+            final Context retval = createmaincall(ret, admin_user, db, access, auth, schemaSelectStrategy);
 
             return retval;
         } catch (final ContextExistsException e) {
-            LOGGER.error("",e);
+            LOGGER.error("", e);
             throw e;
         } catch (final InvalidDataException e) {
             LOGGER.error("", e);
             throw e;
         } catch (StorageException e) {
-            LOGGER.error("", e);
-            // Eliminate nested root cause exceptions. These are mostly unknown to clients.
-            throw new StorageException(e.getMessage());
+            throw e;
         }
     }
 
