@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -127,6 +127,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             throw e3;
         }
 
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
+
         try {
             basicauth.doAuthentication(auth, ctx);
         } catch (final InvalidDataException e) {
@@ -187,7 +191,7 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
 
     @Override
     public void change(final Context ctx, final Group grp, final Credentials credentials) throws RemoteException, StorageException, InvalidCredentialsException, NoSuchContextException, InvalidDataException, DatabaseUpdateException, NoSuchGroupException, NoSuchUserException {
-        final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        Credentials auth = credentials == null ? new Credentials("","") : credentials;
         try {
             doNullCheck(grp);
         } catch (final InvalidDataException e3) {
@@ -195,6 +199,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             throw e3;
         }
 
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
+
         try {
             basicauth.doAuthentication(auth, ctx);
         } catch (final InvalidDataException e1) {
@@ -324,6 +332,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             throw e3;
         }
 
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
+
         try {
             basicauth.doAuthentication(auth, ctx);
         } catch (final InvalidDataException e2) {
@@ -457,6 +469,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             throw e3;
         }
 
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
+
         try {
             basicauth.doAuthentication(auth, ctx);
 
@@ -567,6 +583,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             throw e3;
         }
 
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
+
         try {
             basicauth.doAuthentication(auth, ctx);
         } catch (final InvalidDataException e) {
@@ -619,6 +639,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             throw e3;
         }
 
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
+
         try {
             basicauth.doAuthentication(auth, ctx);
             LOGGER.debug("{} - {} - {}", ctx, Arrays.toString(groups), auth);
@@ -686,6 +710,9 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             InvalidCredentialsException, NoSuchContextException,
             InvalidDataException, DatabaseUpdateException {
         final Credentials auth = credentials == null ? new Credentials("","") : credentials;
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
         basicauth.doAuthentication(auth, ctx);
         LOGGER.debug("{} - {}", ctx, auth);
         checkContextAndSchema(ctx);
@@ -712,6 +739,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             throw e3;
         }
 
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
+
         try {
             basicauth.doAuthentication(auth, ctx);
         } catch (final InvalidDataException e) {
@@ -751,6 +782,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             throw e3;
         }
 
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
+
         try {
             basicauth.doAuthentication(auth, ctx);
         } catch (final InvalidDataException e) {
@@ -787,6 +822,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             throw e3;
         }
 
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
+
         try {
             basicauth.doAuthentication(auth, ctx);
         } catch (final InvalidDataException e) {
@@ -824,6 +863,10 @@ public class OXGroup extends OXCommonImpl implements OXGroupInterface {
             throw e3;
         }
 
+        if (prop.getUserProp(AdminProperties.User.AUTO_LOWERCASE, false)) {
+            auth.setLogin(auth.getLogin().toLowerCase());
+        }
+
         try {
             basicauth.doAuthentication(auth, ctx);
         } catch (final InvalidDataException e) {
