@@ -61,8 +61,9 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Collection;
-import java.util.HashSet;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.SortedMap;
 import java.util.TreeMap;
 import com.openexchange.admin.daemons.ClientAdminThread;
@@ -70,6 +71,8 @@ import com.openexchange.admin.properties.AdminProperties;
 import com.openexchange.admin.rmi.dataobjects.Context;
 import com.openexchange.admin.rmi.dataobjects.Database;
 import com.openexchange.admin.rmi.dataobjects.MaintenanceReason;
+import com.openexchange.admin.rmi.exceptions.ContextExistsException;
+import com.openexchange.admin.rmi.exceptions.InvalidDataException;
 import com.openexchange.admin.rmi.exceptions.PoolException;
 import com.openexchange.admin.rmi.exceptions.StorageException;
 import com.openexchange.admin.services.AdminServiceRegistry;
@@ -78,7 +81,8 @@ import com.openexchange.admin.storage.sqlStorage.OXAdminPoolInterface;
 import com.openexchange.admin.tools.AdminCache;
 import com.openexchange.admin.tools.PropertyHandler;
 import com.openexchange.config.ConfigurationService;
-import com.openexchange.database.Assignment;
+import com.openexchange.database.AssignmentInsertData;
+import com.openexchange.database.Databases;
 import com.openexchange.exception.OXException;
 import com.openexchange.tools.file.external.QuotaFileStorages;
 import com.openexchange.tools.pipesnfilters.DataSource;
@@ -92,8 +96,6 @@ public class OXContextMySQLStorageCommon {
 
     private static final org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(OXContextMySQLStorageCommon.class);
 
-    private final OXUtilMySQLStorageCommon oxutilcommon;
-
     private static AdminCache cache = null;
 
     private static PropertyHandler prop = null;
@@ -103,8 +105,57 @@ public class OXContextMySQLStorageCommon {
         prop = cache.getProperties();
     }
 
+    private static interface StartNumberProvider {
+
+        int getStartValue();
+    }
+
+
+    // -----------------------------------------------------------------------------------------------------------------------------
+
+    private final OXUtilMySQLStorageCommon oxutilcommon;
+    private final Map<String, StartNumberProvider> startValues;
+
     public OXContextMySQLStorageCommon() {
+        super();
         oxutilcommon = new OXUtilMySQLStorageCommon();
+
+        Map<String, StartNumberProvider> startValues = new HashMap<String, StartNumberProvider>(4);
+        startValues.put("sequence_folder", new StartNumberProvider() {
+
+            @Override
+            public int getStartValue() {
+                // below id 20 is reserved
+                return 20;
+            }
+        });
+        startValues.put("sequence_uid_number", new StartNumberProvider() {
+
+            @Override
+            public int getStartValue() {
+                int startnum = Integer.parseInt(prop.getUserProp(AdminProperties.User.UID_NUMBER_START, "-1"));
+                if (startnum > 0) {
+                    // we use the uid number feature
+                    // set the start number in the sequence for uid_numbers
+                    return startnum;
+                }
+                return 0;
+            }
+        });
+        startValues.put("sequence_gid_number", new StartNumberProvider() {
+
+            @Override
+            public int getStartValue() {
+                int startnum = Integer.parseInt(prop.getGroupProp(AdminProperties.Group.GID_NUMBER_START, "-1"));
+                if (startnum > 0) {
+                    // we use the gid number feature
+                    // set the start number in the sequence for gid_numbers
+                    return startnum;
+                }
+                return 0;
+            }
+        });
+        this.startValues = startValues;
     }
 
     // TODO: The average size parameter can be removed if we have an new property handler which can
@@ -206,12 +257,12 @@ public class OXContextMySQLStorageCommon {
         } catch (OXException e) {
             throw new StorageException(e.getMessage(), e);
         } finally {
-            closePreparedStatement(prep);
+            Databases.closeSQLStuff(prep);
             if (oxdb_read != null) {
                 try {
-                        cache.pushConnectionForContextAfterReading(context_id, oxdb_read);
+                    cache.pushConnectionForContextAfterReading(context_id, oxdb_read);
                 } catch (final PoolException exp) {
-                    log.error("Pool Error pushing ox read connection to pool!",exp);
+                    log.error("Pool Error pushing ox read connection to pool!", exp);
                 }
             }
         }
@@ -332,7 +383,7 @@ public class OXContextMySQLStorageCommon {
         }
         try {
             startTransaction(con);
-            cache.getPool().lock(con);
+            cache.getPool().lock(con, poolId);
             deleteEmptySchema(con, poolId, dbSchema);
             con.commit();
         } catch (SQLException e) {
@@ -368,7 +419,7 @@ public class OXContextMySQLStorageCommon {
         if (otherContexts.length == 0) {
             Database db = OXToolStorageInterface.getInstance().loadDatabaseById(poolId);
             db.setScheme(dbSchema);
-            OXUtilMySQLStorageCommon.deleteDatabase(db);
+            OXUtilMySQLStorageCommon.deleteDatabase(db,con);
         }
     }
 
@@ -378,21 +429,38 @@ public class OXContextMySQLStorageCommon {
         try {
             // This creates a lock on context_server2db_pool on the rows with contexts in the same schema. Concurrent create and delete of
             // context can cause removed schemas while creating a context in it. This can not happen anymore with the introduced lock.
-            pool.lock(con);
-            final int poolId = pool.getWritePool(contextId);
-            final String dbSchema = pool.getSchemaName(contextId);
+            int poolId = pool.getWritePool(contextId);
+            String dbSchema = pool.getSchemaName(contextId);
+            int filestoreId = getFilestoreIdFor(con, contextId);
+
+            pool.lock(con, poolId);
+
+            if (0 != filestoreId) {
+                stmt = con.prepareStatement("UPDATE contexts_per_filestore SET count=count-1 WHERE filestore_id=?");
+                stmt.setInt(1, filestoreId);
+                stmt.executeUpdate();
+                Databases.closeSQLStuff(stmt);
+                stmt = null;
+            }
+
+            // Delete association from "context_server2db_pool" table
             pool.deleteAssignment(con, contextId);
+
             deleteEmptySchema(con, poolId, dbSchema);
+
             log.debug("Deleting login2context entries for context {}", I(contextId));
             stmt = con.prepareStatement("DELETE FROM login2context WHERE cid=?");
             stmt.setInt(1, contextId);
             stmt.executeUpdate();
-            stmt.close();
+            Databases.closeSQLStuff(stmt);
+            stmt = null;
+
             log.debug("Deleting context entry for context {}", I(contextId));
             stmt = con.prepareStatement("DELETE FROM context WHERE cid=?");
             stmt.setInt(1, contextId);
             stmt.executeUpdate();
-            stmt.close();
+            Databases.closeSQLStuff(stmt);
+            stmt = null;
         } catch (PoolException e) {
             log.error(e.getMessage(), e);
             throw new StorageException(e.getMessage(), e);
@@ -400,7 +468,24 @@ public class OXContextMySQLStorageCommon {
             log.error(e.getMessage(), e);
             throw new StorageException(e.getMessage(), e);
         } finally {
-            closePreparedStatement(stmt);
+            Databases.closeSQLStuff(stmt);
+        }
+    }
+
+    private int getFilestoreIdFor(Connection con, int contextId) throws SQLException {
+        PreparedStatement stmt = null;
+        ResultSet rs = null;
+        try {
+            stmt = con.prepareStatement("SELECT filestore_id FROM context WHERE cid=?");
+            stmt.setInt(1, contextId);
+            rs = stmt.executeQuery();
+            int filestoreId = 0;
+            if (rs.next()) {
+                filestoreId = rs.getInt(1);
+            }
+            return filestoreId;
+        } finally {
+            Databases.closeSQLStuff(rs, stmt);
         }
     }
 
@@ -421,7 +506,17 @@ public class OXContextMySQLStorageCommon {
         return tmp.toArray(new String[tmp.size()]);
     }
 
-    public void fillContextAndServer2DBPool(final Context ctx, final Connection con, final Database db) throws StorageException {
+    /**
+     * Inserts context data to appropriate tables.
+     *
+     * @param ctx The context to add
+     * @param con The connection to the configdb
+     * @param db The database associated with the context
+     * @throws StorageException If a general storage error occurs
+     * @throws ContextExistsException If there is already a context with the same context identifier
+     * @throws InvalidDataException If there is already a context with the same name
+     */
+    public void fillContextAndServer2DBPool(final Context ctx, final Connection con, final Database db) throws StorageException, ContextExistsException, InvalidDataException {
         // dbid is the id in db_pool of database engine to use for next context
 
         // if read id -1 (not set by client ) or 0 (there is no read db for this
@@ -431,10 +526,11 @@ public class OXContextMySQLStorageCommon {
             db.setRead_id(db.getId());
         }
         fillContextTable(ctx, con);
+        incrementContextsPerFilestoreTable(ctx, con);
 
         try {
             final int serverId = ClientAdminThread.cache.getServerId();
-            ClientAdminThread.cache.getPool().writeAssignment(con, new Assignment() {
+            ClientAdminThread.cache.getPool().writeAssignment(con, new AssignmentInsertData() {
                 @Override
                 public int getContextId() {
                     return i(ctx.getId());
@@ -461,6 +557,18 @@ public class OXContextMySQLStorageCommon {
         }
     }
 
+    public final void handleCreateContextRollback(Connection configCon, int contextId) throws StorageException {
+        // remove all entries from configuration database because everything to configuration database has been committed.
+        try {
+            if (configCon != null) {
+                deleteContextFromConfigDB(configCon, contextId);
+                configCon.commit();
+            }
+        } catch (final SQLException e) {
+            log.error("SQL Error removing/rollback entries from configdb for context {}", contextId, e);
+        }
+    }
+
     public final void handleContextDeleteRollback(final Connection write_ox_con, final Connection con_write) {
         try {
             if (con_write != null && !con_write.getAutoCommit()) {
@@ -519,42 +627,32 @@ public class OXContextMySQLStorageCommon {
     }
 
     private int modifyStartValue(final String tableName) {
-        int retval = 0;
-        if ("sequence_folder".equals(tableName)) {
-            // below id 20 is reserved
-            retval = 20;
-        }
-        // check for the uid number feature
-        if ("sequence_uid_number".equals(tableName)) {
-            final int startnum = Integer.parseInt(prop.getUserProp(AdminProperties.User.UID_NUMBER_START, "-1"));
-            if (startnum > 0) {
-                // we use the uid number feature
-                // set the start number in the sequence for uid_numbers
-                retval = startnum;
-            }
-        }
-        // check for the gid number feature
-        if ("sequence_gid_number".equals(tableName)){
-            final int startnum = Integer.parseInt(prop.getGroupProp(AdminProperties.Group.GID_NUMBER_START, "-1"));
-            if (startnum > 0) {
-                // we use the gid number feature
-                // set the start number in the sequence for gid_numbers
-                retval = startnum;
-            }
-        }
-        return retval;
+        StartNumberProvider startNumberProvider = startValues.get(tableName);
+        return null == startNumberProvider ? 0 : startNumberProvider.getStartValue();
     }
 
-    private final void fillContextTable(final Context ctx, final Connection configdbCon) throws StorageException {
+    /**
+     * <code>INSERT</code>s the data row into the "context" table.
+     *
+     * @param ctx The context to insert
+     * @param configdbCon A connection to the configdb
+     * @throws StorageException If a general SQL error occurs
+     * @throws ContextExistsException If there is already a context with the same context identifier
+     * @throws InvalidDataException If there is already a context with the same name
+     */
+    private final void fillContextTable(final Context ctx, final Connection configdbCon) throws StorageException, ContextExistsException, InvalidDataException {
+        String name;
+        if (ctx.getName() != null && ctx.getName().trim().length() > 0) {
+            name = ctx.getName();
+        } else {
+            name = ctx.getIdAsString();
+        }
+
         PreparedStatement stmt = null;
         try {
             stmt = configdbCon.prepareStatement("INSERT INTO context (cid,name,enabled,filestore_id,filestore_name,quota_max) VALUES (?,?,?,?,?,?)");
             stmt.setInt(1, ctx.getId().intValue());
-            if (ctx.getName() != null && ctx.getName().trim().length() > 0) {
-                stmt.setString(2, ctx.getName());
-            } else {
-                stmt.setString(2, ctx.getIdAsString());
-            }
+            stmt.setString(2, name);
             stmt.setBoolean(3, true);
             stmt.setInt(4, ctx.getFilestoreId().intValue());
             stmt.setString(5, ctx.getFilestore_name());
@@ -566,65 +664,63 @@ public class OXContextMySQLStorageCommon {
             stmt.setLong(6, quota_max_temp);
             stmt.executeUpdate();
         } catch (final SQLException e) {
+            if (Databases.isPrimaryKeyConflictInMySQL(e)) {
+                throw new ContextExistsException("Context " + ctx.getId().intValue() + " already exists!");
+            }
+            if (Databases.isKeyConflictInMySQL(e, "context_name_unique")) {
+                throw new InvalidDataException("Context " + name + " already exists!");
+            }
             throw new StorageException(e.getMessage(), e);
         } finally {
-            closePreparedStatement(stmt);
+            Databases.closeSQLStuff(stmt);
         }
     }
 
-    public void fillLogin2ContextTable(final Context ctx, final Connection configdb_write_con) throws SQLException, StorageException {
-        final HashSet<String> loginMappings = ctx.getLoginMappings();
-        final Integer ctxid = ctx.getId();
+    /**
+     * <code>INSERT</code>s the data row into the "context" table.
+     *
+     * @param ctx The context newly using a filestore
+     * @param configdbCon A connection to the configdb
+     * @throws StorageException If a general SQL error occurs
+     */
+    private final void incrementContextsPerFilestoreTable(final Context ctx, final Connection configdbCon) throws StorageException {
         PreparedStatement stmt = null;
-        PreparedStatement checkAvailable = null;
-        ResultSet found = null;
         try {
-            checkAvailable = configdb_write_con.prepareStatement("SELECT 1 FROM login2context WHERE login_info = ?");
-            stmt = configdb_write_con.prepareStatement("INSERT INTO login2context (cid,login_info) VALUES (?,?)");
-            for (final String mapping : loginMappings) {
-                if (null != mapping) {
-                    checkAvailable.setString(1, mapping);
-                    found = checkAvailable.executeQuery();
-                    final boolean mappingTaken = found.next();
-                    found.close();
-
-                    if(mappingTaken) {
-                        throw new StorageException("Cannot map '"+mapping+"' to the newly created context. This mapping is already in use.");
-                    }
-
-                    stmt.setInt(1, ctxid.intValue());
-                    stmt.setString(2, mapping);
-                    stmt.executeUpdate();
-                }
-            }
-        } catch (final SQLException sql) {
-            log.error("SQL Error", sql);
-            throw sql;
+            stmt = configdbCon.prepareStatement("UPDATE contexts_per_filestore SET count=count+1 WHERE filestore_id=?");
+            stmt.setInt(1, ctx.getFilestoreId().intValue());
+            stmt.executeUpdate();
+        } catch (final SQLException e) {
+            throw new StorageException(e.getMessage(), e);
         } finally {
-            closeResultSet(found);
-            closePreparedStatement(checkAvailable);
-            closePreparedStatement(stmt);
-
+            Databases.closeSQLStuff(stmt);
         }
     }
 
-    private void closeResultSet(final ResultSet rs) {
-        if (rs != null) {
-            try {
-                    rs.close();
-            } catch (final SQLException e) {
-                log.error(OXContextMySQLStorageCommon.LOG_ERROR_CLOSING_STATEMENT, e);
+    public void fillLogin2ContextTable(Context ctx, Connection configdb_write_con) throws SQLException, StorageException {
+        int contextId = ctx.getId().intValue();
+        for (String mapping : ctx.getLoginMappings()) {
+            if (null != mapping) {
+                insertLogin2ContextMapping(mapping, contextId, configdb_write_con);
             }
         }
     }
 
-    private void closePreparedStatement(final PreparedStatement stmt) {
-        if (stmt != null) {
-            try {
-                    stmt.close();
-            } catch (final SQLException e) {
-                log.error(OXContextMySQLStorageCommon.LOG_ERROR_CLOSING_STATEMENT, e);
+    private void insertLogin2ContextMapping(String mapping, int contextId, Connection configdb_write_con) throws SQLException, StorageException {
+        PreparedStatement stmt = null;
+        try {
+            stmt = configdb_write_con.prepareStatement("INSERT INTO login2context (cid,login_info) VALUES (?,?)");
+            stmt.setInt(1, contextId);
+            stmt.setString(2, mapping);
+            stmt.executeUpdate();
+        } catch (final SQLException e) {
+            if (Databases.isPrimaryKeyConflictInMySQL(e)) {
+                throw new StorageException("Cannot map '"+mapping+"' to the newly created context. This mapping is already in use.", e);
             }
+            log.error("SQL Error", e);
+            throw e;
+        } finally {
+            Databases.closeSQLStuff(stmt);
         }
     }
+
 }
