@@ -51,6 +51,7 @@ package com.openexchange.admin.storage.sqlStorage;
 
 import java.sql.Connection;
 import java.sql.SQLException;
+import java.util.Map;
 import com.openexchange.admin.rmi.exceptions.PoolException;
 import com.openexchange.database.Assignment;
 import com.openexchange.database.DatabaseService;
@@ -95,6 +96,16 @@ public class OXAdminPoolDBPool implements OXAdminPoolInterface {
         return con;
     }
 
+    @Override
+    public Connection getConnectionForConfigDBNoTimeout() throws PoolException {
+        try {
+            return getService().getForUpdateTask();
+        } catch (OXException e) {
+            log.error("Error pickup no-timeout configdb database write connection from pool!", e);
+            throw new PoolException(e.getMessage());
+        }
+    }
+
     @Override
     public Connection getConnectionForContext(int contextId) throws PoolException {
         final Connection con;
@@ -133,8 +144,12 @@ public class OXAdminPoolDBPool implements OXAdminPoolInterface {
 
     @Override
     public boolean pushConnectionForConfigDB(Connection con) throws PoolException {
+        if (null == con) {
+            return false;
+        }
+
         try {
-            if (con != null && !con.getAutoCommit() && !con.isClosed()) {
+            if (!con.getAutoCommit() && !con.isClosed()) {
                 con.setAutoCommit(true);
             }
         } catch (SQLException e) {
@@ -147,9 +162,31 @@ public class OXAdminPoolDBPool implements OXAdminPoolInterface {
     }
 
     @Override
-    public boolean pushConnectionForContext(int contextId, Connection con) throws PoolException {
+    public boolean pushConnectionForConfigDBNoTimeout(Connection con) throws PoolException {
+        if (null == con) {
+            return false;
+        }
         try {
-            if (con != null && !con.getAutoCommit() && !con.isClosed()) {
+            if (!con.getAutoCommit() && !con.isClosed()) {
+                con.setAutoCommit(true);
+            }
+        } catch (SQLException e) {
+            log.error("Error pushing no-timeout configdb write connection to pool!", e);
+            throw new PoolException(e.getMessage());
+        } finally {
+            getService().backForUpdateTask(con);
+        }
+        return true;
+    }
+
+    @Override
+    public boolean pushConnectionForContext(int contextId, Connection con) throws PoolException {
+        if (null == con) {
+            return false;
+        }
+
+        try {
+            if (!con.getAutoCommit() && !con.isClosed()) {
                 con.setAutoCommit(true);
             }
         } catch (SQLException e) {
@@ -163,8 +200,12 @@ public class OXAdminPoolDBPool implements OXAdminPoolInterface {
 
     @Override
     public boolean pushConnectionForContextAfterReading(int contextId, Connection con) throws PoolException {
+        if (null == con) {
+            return false;
+        }
+
         try {
-            if (con != null && !con.getAutoCommit() && !con.isClosed()) {
+            if (!con.getAutoCommit() && !con.isClosed()) {
                 con.setAutoCommit(true);
             }
         } catch (SQLException e) {
@@ -178,8 +219,12 @@ public class OXAdminPoolDBPool implements OXAdminPoolInterface {
 
     @Override
     public boolean pushConnectionForContextNoTimeout(int contextId, Connection con) throws PoolException {
+        if (null == con) {
+            return false;
+        }
+
         try {
-            if (null != con && !con.getAutoCommit() && !con.isClosed()) {
+            if (!con.getAutoCommit() && !con.isClosed()) {
                 con.setAutoCommit(true);
             }
         } catch (SQLException e) {
@@ -193,8 +238,12 @@ public class OXAdminPoolDBPool implements OXAdminPoolInterface {
 
     @Override
     public boolean pushConnection(int poolId, Connection con) throws PoolException {
+        if (null == con) {
+            return false;
+        }
+
         try {
-            if (null != con && !con.getAutoCommit() && !con.isClosed()) {
+            if (!con.getAutoCommit() && !con.isClosed()) {
                 con.setAutoCommit(true);
             }
         } catch (SQLException e) {
@@ -278,6 +327,16 @@ public class OXAdminPoolDBPool implements OXAdminPoolInterface {
         }
     }
 
+    @Override
+    public Map<String, Integer> getContextCountPerSchema(Connection con, int poolId, int maxContexts) throws PoolException {
+        try {
+            return getService().getContextCountPerSchema(con, poolId, maxContexts);
+        } catch (OXException e) {
+            log.error("Error getting unfilled schemas", e);
+            throw new PoolException(e.getMessage());
+        }
+    }
+
     @Override
     public int getWritePool(int contextId) throws PoolException {
         try {
@@ -299,11 +358,11 @@ public class OXAdminPoolDBPool implements OXAdminPoolInterface {
     }
 
     @Override
-    public void lock(Connection con) throws PoolException {
+    public void lock(Connection con, int writePoolId) throws PoolException {
         try {
-            getService().lock(con);
+            getService().lock(con, writePoolId);
         } catch (OXException e) {
-            log.error("Error locking context_server2db_pool table", e);
+            log.error("Error locking database pool {}", writePoolId, e);
             throw new PoolException(e.getMessage());
         }
     }
