@@ -50,6 +50,7 @@
 package com.openexchange.admin.storage.sqlStorage;
 
 import java.sql.Connection;
+import java.util.Map;
 import com.openexchange.admin.rmi.exceptions.PoolException;
 import com.openexchange.database.Assignment;
 import com.openexchange.database.DatabaseService;
@@ -60,6 +61,8 @@ public interface OXAdminPoolInterface {
 
     Connection getConnectionForConfigDB() throws PoolException;
 
+    Connection getConnectionForConfigDBNoTimeout() throws PoolException;
+
     Connection getConnectionForContext(int contextId) throws PoolException;
 
     Connection getConnectionForContextNoTimeout(int contextId) throws PoolException;
@@ -68,6 +71,8 @@ public interface OXAdminPoolInterface {
 
     boolean pushConnectionForConfigDB(Connection con) throws PoolException;
 
+    boolean pushConnectionForConfigDBNoTimeout(Connection con) throws PoolException;
+
     boolean pushConnectionForContext(int contextId, Connection con) throws PoolException;
 
     boolean pushConnectionForContextAfterReading(int contextId, Connection con) throws PoolException;
@@ -96,5 +101,16 @@ public interface OXAdminPoolInterface {
 
     String[] getUnfilledSchemas(Connection con, int poolId, int maxContexts) throws PoolException;
 
-    void lock(Connection con) throws PoolException;
+    /**
+     * Gets the number of contexts per schema that are located in given database identified by <code>poolId</code>.
+     *
+     * @param con The connection to the config database
+     * @param poolId The pool identifier
+     * @param maxContexts The configured maximum allowed contexts for a database schema.
+     * @return A mapping providing the count per schema
+     * @throws PoolException If schema count cannot be returned
+     */
+    Map<String, Integer> getContextCountPerSchema(Connection con, int poolId, int maxContexts) throws PoolException;
+
+    void lock(Connection con, int writePoolId) throws PoolException;
 }
